@@ -46,22 +46,22 @@ typedef struct _GstTypeFindFactoryClass GstTypeFindFactoryClass;
 
 /* typefinding interface */
 
-GST_EXPORT
+GST_API
 GType           gst_type_find_factory_get_type          (void);
 
-GST_EXPORT
+GST_API
 GList *         gst_type_find_factory_get_list          (void);
 
-GST_EXPORT
+GST_API
 const gchar * const * gst_type_find_factory_get_extensions (GstTypeFindFactory *factory);
 
-GST_EXPORT
+GST_API
 GstCaps *       gst_type_find_factory_get_caps          (GstTypeFindFactory *factory);
 
-GST_EXPORT
+GST_API
 gboolean        gst_type_find_factory_has_function      (GstTypeFindFactory *factory);
 
-GST_EXPORT
+GST_API
 void            gst_type_find_factory_call_function     (GstTypeFindFactory *factory,
                                                          GstTypeFind *find);
 
