@@ -17,8 +17,9 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-
-#include <unistd.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/check/gstcheck.h>
 
@@ -314,6 +315,8 @@ mq_dummypad_query (GstPad * sinkpad, GstObject * parent, GstQuery * query)
 
 struct PadData
 {
+  GstPad *input_pad;
+  GstPad *out_pad;
   guint8 pad_num;
   guint32 *max_linked_id_ptr;
   guint32 *eos_count_ptr;
@@ -352,20 +355,22 @@ mq_dummypad_chain (GstPad * sinkpad, GstObject * parent, GstBuffer * buf)
   /* For not-linked pads, ensure that we're not running ahead of the 'linked'
    * pads. The first buffer is allowed to get ahead, because otherwise things can't
    * always pre-roll correctly */
-  if (!pad_data->is_linked) {
-    /* If there are no linked pads, we can't track a max_id for them :) */
-    if (pad_data->n_linked > 0 && !pad_data->first_buf) {
-      g_mutex_lock (&_check_lock);
-      fail_unless (cur_id <= *(pad_data->max_linked_id_ptr) + 1,
-          "Got buffer %u on pad %u before buffer %u was seen on a "
-          "linked pad (max: %u)", cur_id, pad_data->pad_num, cur_id - 1,
-          *(pad_data->max_linked_id_ptr));
-      g_mutex_unlock (&_check_lock);
+  if (pad_data->max_linked_id_ptr) {
+    if (!pad_data->is_linked) {
+      /* If there are no linked pads, we can't track a max_id for them :) */
+      if (pad_data->n_linked > 0 && !pad_data->first_buf) {
+        g_mutex_lock (&_check_lock);
+        fail_unless (cur_id <= *(pad_data->max_linked_id_ptr) + 1,
+            "Got buffer %u on pad %u before buffer %u was seen on a "
+            "linked pad (max: %u)", cur_id, pad_data->pad_num, cur_id - 1,
+            *(pad_data->max_linked_id_ptr));
+        g_mutex_unlock (&_check_lock);
+      }
+    } else {
+      /* Update the max_id value */
+      if (cur_id > *(pad_data->max_linked_id_ptr))
+        *(pad_data->max_linked_id_ptr) = cur_id;
     }
-  } else {
-    /* Update the max_id value */
-    if (cur_id > *(pad_data->max_linked_id_ptr))
-      *(pad_data->max_linked_id_ptr) = cur_id;
   }
   pad_data->first_buf = FALSE;
 
@@ -405,17 +410,113 @@ mq_dummypad_event (GstPad * sinkpad, GstObject * parent, GstEvent * event)
   return TRUE;
 }
 
+static void
+construct_n_pads (GstElement * mq, struct PadData *pad_data, gint n_pads,
+    gint n_linked)
+{
+  gint i;
+  GstSegment segment;
+
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+
+  /* Construct NPADS dummy output pads. The first 'n_linked' return FLOW_OK, the rest
+   * return NOT_LINKED. The not-linked ones check the expected ordering of 
+   * output buffers */
+  for (i = 0; i < n_pads; i++) {
+    GstPad *mq_srcpad, *mq_sinkpad, *inpad, *outpad;
+    gchar *name;
+
+    name = g_strdup_printf ("dummysrc%d", i);
+    inpad = gst_pad_new (name, GST_PAD_SRC);
+    g_free (name);
+    gst_pad_set_query_function (inpad, mq_dummypad_query);
+
+    mq_sinkpad = gst_element_get_request_pad (mq, "sink_%u");
+    fail_unless (mq_sinkpad != NULL);
+    fail_unless (gst_pad_link (inpad, mq_sinkpad) == GST_PAD_LINK_OK);
+
+    gst_pad_set_active (inpad, TRUE);
+
+    gst_pad_push_event (inpad, gst_event_new_stream_start ("test"));
+    gst_pad_push_event (inpad, gst_event_new_segment (&segment));
+
+    mq_srcpad = mq_sinkpad_to_srcpad (mq, mq_sinkpad);
+
+    name = g_strdup_printf ("dummysink%d", i);
+    outpad = gst_pad_new (name, GST_PAD_SINK);
+    g_free (name);
+    gst_pad_set_chain_function (outpad, mq_dummypad_chain);
+    gst_pad_set_event_function (outpad, mq_dummypad_event);
+    gst_pad_set_query_function (outpad, mq_dummypad_query);
+
+    pad_data[i].pad_num = i;
+    pad_data[i].input_pad = inpad;
+    pad_data[i].out_pad = outpad;
+    pad_data[i].max_linked_id_ptr = NULL;
+    pad_data[i].eos_count_ptr = NULL;
+    pad_data[i].is_linked = (i < n_linked ? TRUE : FALSE);
+    pad_data[i].n_linked = n_linked;
+    pad_data[i].cond = NULL;
+    pad_data[i].mutex = NULL;
+    pad_data[i].first_buf = TRUE;
+    gst_pad_set_element_private (outpad, pad_data + i);
+
+    fail_unless (gst_pad_link (mq_srcpad, outpad) == GST_PAD_LINK_OK);
+    gst_pad_set_active (outpad, TRUE);
+
+    gst_object_unref (mq_sinkpad);
+    gst_object_unref (mq_srcpad);
+  }
+}
+
+static void
+push_n_buffers (struct PadData *pad_data, gint num_buffers,
+    const guint8 * pad_pattern, guint pattern_size)
+{
+  gint i;
+
+  for (i = 0; i < num_buffers; i++) {
+    guint8 cur_pad;
+    GstBuffer *buf;
+    GstFlowReturn ret;
+    GstMapInfo info;
+
+    cur_pad = pad_pattern[i % pattern_size];
+
+    buf = gst_buffer_new_and_alloc (4);
+    g_mutex_lock (&_check_lock);
+    fail_if (buf == NULL);
+    g_mutex_unlock (&_check_lock);
+
+    fail_unless (gst_buffer_map (buf, &info, GST_MAP_WRITE));
+    GST_WRITE_UINT32_BE (info.data, i + 1);
+    gst_buffer_unmap (buf, &info);
+    GST_BUFFER_TIMESTAMP (buf) = (i + 1) * GST_SECOND;
+
+    ret = gst_pad_push (pad_data[cur_pad].input_pad, buf);
+    g_mutex_lock (&_check_lock);
+    if (pad_data[cur_pad].is_linked) {
+      fail_unless (ret == GST_FLOW_OK,
+          "Push on pad %d returned %d when FLOW_OK was expected", cur_pad, ret);
+    } else {
+      /* Expect OK initially, then NOT_LINKED when the srcpad starts pushing */
+      fail_unless (ret == GST_FLOW_OK || ret == GST_FLOW_NOT_LINKED,
+          "Push on pad %d returned %d when FLOW_OK or NOT_LINKED  was expected",
+          cur_pad, ret);
+    }
+    g_mutex_unlock (&_check_lock);
+  }
+}
+
 static void
 run_output_order_test (gint n_linked)
 {
-  /* This test creates a multiqueue with 2 linked output, and 3 outputs that 
-   * return 'not-linked' when data is pushed, then verifies that all buffers 
-   * are received on not-linked pads only after earlier buffers on the 
+  /* This test creates a multiqueue with 2 linked output, and 3 outputs that
+   * return 'not-linked' when data is pushed, then verifies that all buffers
+   * are received on not-linked pads only after earlier buffers on the
    * 'linked' pads are made */
   GstElement *pipe;
   GstElement *mq;
-  GstPad *inputpads[5];
-  GstPad *sinkpads[5];
   struct PadData pad_data[5];
   guint32 max_linked_id;
   guint32 eos_seen;
@@ -424,6 +525,99 @@ run_output_order_test (gint n_linked)
   gint i;
   const gint NPADS = 5;
   const gint NBUFFERS = 1000;
+
+  g_mutex_init (&mutex);
+  g_cond_init (&cond);
+
+  pipe = gst_bin_new ("testbin");
+
+  mq = gst_element_factory_make ("multiqueue", NULL);
+  fail_unless (mq != NULL);
+  gst_bin_add (GST_BIN (pipe), mq);
+
+  /* No limits */
+  g_object_set (mq,
+      "max-size-bytes", (guint) 0,
+      "max-size-buffers", (guint) 0,
+      "max-size-time", (guint64) 0,
+      "extra-size-bytes", (guint) 0,
+      "extra-size-buffers", (guint) 0, "extra-size-time", (guint64) 0, NULL);
+
+  construct_n_pads (mq, pad_data, NPADS, n_linked);
+  for (i = 0; i < NPADS; i++) {
+    pad_data[i].max_linked_id_ptr = &max_linked_id;
+    /* Only look for EOS on the linked pads */
+    pad_data[i].eos_count_ptr = (i < n_linked) ? &eos_seen : NULL;
+    pad_data[i].cond = &cond;
+    pad_data[i].mutex = &mutex;
+  }
+
+  /* Run the test. Push 1000 buffers through the multiqueue in a pattern */
+  max_linked_id = 0;
+  eos_seen = 0;
+  gst_element_set_state (pipe, GST_STATE_PLAYING);
+
+  {
+    const guint8 pad_pattern[] =
+        { 0, 0, 0, 0, 1, 1, 2, 1, 0, 2, 3, 2, 3, 1, 4 };
+    const guint n = sizeof (pad_pattern) / sizeof (guint8);
+    push_n_buffers (pad_data, NBUFFERS, pad_pattern, n);
+  }
+
+  for (i = 0; i < NPADS; i++) {
+    gst_pad_push_event (pad_data[i].input_pad, gst_event_new_eos ());
+  }
+
+  /* Wait while the buffers are processed */
+  g_mutex_lock (&mutex);
+  /* We wait until EOS has been pushed on all linked pads */
+  while (eos_seen < n_linked) {
+    g_cond_wait (&cond, &mutex);
+  }
+  g_mutex_unlock (&mutex);
+
+  /* Clean up */
+  for (i = 0; i < NPADS; i++) {
+    GstPad *mq_input = gst_pad_get_peer (pad_data[i].input_pad);
+
+    gst_pad_unlink (pad_data[i].input_pad, mq_input);
+    gst_element_release_request_pad (mq, mq_input);
+    gst_object_unref (mq_input);
+    gst_object_unref (pad_data[i].input_pad);
+    gst_object_unref (pad_data[i].out_pad);
+  }
+
+  gst_element_set_state (pipe, GST_STATE_NULL);
+  gst_object_unref (pipe);
+
+  g_cond_clear (&cond);
+  g_mutex_clear (&mutex);
+}
+
+GST_START_TEST (test_output_order)
+{
+  run_output_order_test (2);
+  run_output_order_test (0);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_not_linked_eos)
+{
+  /* This test creates a multiqueue with 1 linked output and 1 not-linked
+   * pad. It pushes a few buffers through each, then EOS on the linked
+   * pad and waits until that arrives. After that, it pushes some more
+   * buffers on the not-linked pad and then EOS and checks that those
+   * are all output */
+  GstElement *pipe;
+  GstElement *mq;
+  struct PadData pad_data[2];
+  guint32 eos_seen;
+  GMutex mutex;
+  GCond cond;
+  gint i;
+  const gint NPADS = 2;
+  const gint NBUFFERS = 20;
   GstSegment segment;
 
   gst_segment_init (&segment, GST_FORMAT_BYTES);
@@ -445,115 +639,58 @@ run_output_order_test (gint n_linked)
       "extra-size-bytes", (guint) 0,
       "extra-size-buffers", (guint) 0, "extra-size-time", (guint64) 0, NULL);
 
-  /* Construct NPADS dummy output pads. The first 'n_linked' return FLOW_OK, the rest
-   * return NOT_LINKED. The not-linked ones check the expected ordering of 
-   * output buffers */
+  /* Construct NPADS dummy output pads. The first 1 returns FLOW_OK, the rest
+   * return NOT_LINKED. */
+  construct_n_pads (mq, pad_data, NPADS, 1);
   for (i = 0; i < NPADS; i++) {
-    GstPad *mq_srcpad, *mq_sinkpad;
-    gchar *name;
-
-    name = g_strdup_printf ("dummysrc%d", i);
-    inputpads[i] = gst_pad_new (name, GST_PAD_SRC);
-    g_free (name);
-    gst_pad_set_query_function (inputpads[i], mq_dummypad_query);
-
-    mq_sinkpad = gst_element_get_request_pad (mq, "sink_%u");
-    fail_unless (mq_sinkpad != NULL);
-    fail_unless (gst_pad_link (inputpads[i], mq_sinkpad) == GST_PAD_LINK_OK);
-
-    gst_pad_set_active (inputpads[i], TRUE);
-
-    gst_pad_push_event (inputpads[i], gst_event_new_stream_start ("test"));
-    gst_pad_push_event (inputpads[i], gst_event_new_segment (&segment));
-
-    mq_srcpad = mq_sinkpad_to_srcpad (mq, mq_sinkpad);
-
-    name = g_strdup_printf ("dummysink%d", i);
-    sinkpads[i] = gst_pad_new (name, GST_PAD_SINK);
-    g_free (name);
-    gst_pad_set_chain_function (sinkpads[i], mq_dummypad_chain);
-    gst_pad_set_event_function (sinkpads[i], mq_dummypad_event);
-    gst_pad_set_query_function (sinkpads[i], mq_dummypad_query);
-
-    pad_data[i].pad_num = i;
-    pad_data[i].max_linked_id_ptr = &max_linked_id;
+    /* Only look for EOS on the linked pads */
     pad_data[i].eos_count_ptr = &eos_seen;
-    pad_data[i].is_linked = (i < n_linked ? TRUE : FALSE);
-    pad_data[i].n_linked = n_linked;
     pad_data[i].cond = &cond;
     pad_data[i].mutex = &mutex;
-    pad_data[i].first_buf = TRUE;
-    gst_pad_set_element_private (sinkpads[i], pad_data + i);
-
-    fail_unless (gst_pad_link (mq_srcpad, sinkpads[i]) == GST_PAD_LINK_OK);
-    gst_pad_set_active (sinkpads[i], TRUE);
-
-    gst_object_unref (mq_sinkpad);
-    gst_object_unref (mq_srcpad);
   }
 
-  /* Run the test. Push 1000 buffers through the multiqueue in a pattern */
-
-  max_linked_id = 0;
+  /* Run the test. Push 20 buffers through the multiqueue in a pattern */
   eos_seen = 0;
   gst_element_set_state (pipe, GST_STATE_PLAYING);
 
-  for (i = 0; i < NBUFFERS; i++) {
-    const guint8 pad_pattern[] =
-        { 0, 0, 0, 0, 1, 1, 2, 1, 0, 2, 3, 2, 3, 1, 4 };
+  {
+    const guint8 pad_pattern[] = { 0, 1 };
     const guint n = sizeof (pad_pattern) / sizeof (guint8);
-    guint8 cur_pad;
-    GstBuffer *buf;
-    GstFlowReturn ret;
-    GstMapInfo info;
-
-    cur_pad = pad_pattern[i % n];
-
-    buf = gst_buffer_new_and_alloc (4);
-    g_mutex_lock (&_check_lock);
-    fail_if (buf == NULL);
-    g_mutex_unlock (&_check_lock);
-
-    fail_unless (gst_buffer_map (buf, &info, GST_MAP_WRITE));
-    GST_WRITE_UINT32_BE (info.data, i + 1);
-    gst_buffer_unmap (buf, &info);
-    GST_BUFFER_TIMESTAMP (buf) = (i + 1) * GST_SECOND;
-
-    ret = gst_pad_push (inputpads[cur_pad], buf);
-    g_mutex_lock (&_check_lock);
-    if (pad_data[cur_pad].is_linked) {
-      fail_unless (ret == GST_FLOW_OK,
-          "Push on pad %d returned %d when FLOW_OK was expected", cur_pad, ret);
-    } else {
-      /* Expect OK initially, then NOT_LINKED when the srcpad starts pushing */
-      fail_unless (ret == GST_FLOW_OK || ret == GST_FLOW_NOT_LINKED,
-          "Push on pad %d returned %d when FLOW_OK or NOT_LINKED  was expected",
-          cur_pad, ret);
-    }
-    g_mutex_unlock (&_check_lock);
-  }
-  for (i = 0; i < NPADS; i++) {
-    gst_pad_push_event (inputpads[i], gst_event_new_eos ());
+    push_n_buffers (pad_data, NBUFFERS, pad_pattern, n);
   }
 
-  /* Wait while the buffers are processed */
+  /* Make the linked pad go EOS */
+  gst_pad_push_event (pad_data[0].input_pad, gst_event_new_eos ());
+
   g_mutex_lock (&mutex);
-  /* We wait until EOS has been pushed on all linked pads */
-  while (eos_seen < n_linked) {
+  /* Wait until EOS has been seen on the linked pad */
+  while (eos_seen == 0)
     g_cond_wait (&cond, &mutex);
+  g_mutex_unlock (&mutex);
+
+  /* Now push some more buffers to the not-linked pad */
+  {
+    const guint8 pad_pattern[] = { 1, 1 };
+    const guint n = sizeof (pad_pattern) / sizeof (guint8);
+    push_n_buffers (pad_data, NBUFFERS, pad_pattern, n);
   }
+  /* And EOS on the not-linked pad */
+  gst_pad_push_event (pad_data[1].input_pad, gst_event_new_eos ());
+
+  g_mutex_lock (&mutex);
+  while (eos_seen < NPADS)
+    g_cond_wait (&cond, &mutex);
   g_mutex_unlock (&mutex);
 
   /* Clean up */
   for (i = 0; i < NPADS; i++) {
-    GstPad *mq_input = gst_pad_get_peer (inputpads[i]);
+    GstPad *mq_input = gst_pad_get_peer (pad_data[i].input_pad);
 
-    gst_pad_unlink (inputpads[i], mq_input);
+    gst_pad_unlink (pad_data[i].input_pad, mq_input);
     gst_element_release_request_pad (mq, mq_input);
     gst_object_unref (mq_input);
-    gst_object_unref (inputpads[i]);
-
-    gst_object_unref (sinkpads[i]);
+    gst_object_unref (pad_data[i].input_pad);
+    gst_object_unref (pad_data[i].out_pad);
   }
 
   gst_element_set_state (pipe, GST_STATE_NULL);
@@ -563,12 +700,6 @@ run_output_order_test (gint n_linked)
   g_mutex_clear (&mutex);
 }
 
-GST_START_TEST (test_output_order)
-{
-  run_output_order_test (2);
-  run_output_order_test (0);
-}
-
 GST_END_TEST;
 
 GST_START_TEST (test_sparse_stream)
@@ -732,6 +863,415 @@ GST_START_TEST (test_sparse_stream)
 
 GST_END_TEST;
 
+static gpointer
+pad_push_datablock_thread (gpointer data)
+{
+  GstPad *pad = data;
+  GstBuffer *buf;
+
+  buf = gst_buffer_new_allocate (NULL, 80 * 1000, NULL);
+  gst_pad_push (pad, buf);
+
+  return NULL;
+}
+
+static GstPadProbeReturn
+block_probe (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
+{
+  return GST_PAD_PROBE_OK;
+}
+
+static void
+check_for_buffering_msg (GstElement * pipeline, gint expected_perc)
+{
+  gint buf_perc;
+  GstMessage *msg;
+
+  GST_LOG ("waiting for %d%% buffering message", expected_perc);
+
+  msg = gst_bus_poll (GST_ELEMENT_BUS (pipeline),
+      GST_MESSAGE_BUFFERING | GST_MESSAGE_ERROR, -1);
+  fail_if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR,
+      "Expected BUFFERING message, got ERROR message");
+
+  gst_message_parse_buffering (msg, &buf_perc);
+  fail_unless (buf_perc == expected_perc,
+      "Got incorrect percentage: %d%% expected: %d%%", buf_perc, expected_perc);
+
+  gst_message_unref (msg);
+}
+
+GST_START_TEST (test_initial_fill_above_high_threshold)
+{
+  /* This test checks what happens if the first buffer that enters
+   * the queue immediately fills it above the high-threshold. */
+  GstElement *pipe;
+  GstElement *mq, *fakesink;
+  GstPad *inputpad;
+  GstPad *mq_sinkpad;
+  GstPad *sinkpad;
+  GstSegment segment;
+  GThread *thread;
+
+
+  /* Setup test pipeline with one multiqueue and one fakesink */
+
+  pipe = gst_pipeline_new ("testbin");
+  mq = gst_element_factory_make ("multiqueue", NULL);
+  fail_unless (mq != NULL);
+  gst_bin_add (GST_BIN (pipe), mq);
+
+  fakesink = gst_element_factory_make ("fakesink", NULL);
+  fail_unless (fakesink != NULL);
+  gst_bin_add (GST_BIN (pipe), fakesink);
+
+  /* Block fakesink sinkpad flow to ensure the queue isn't emptied
+   * by the prerolling sink */
+  sinkpad = gst_element_get_static_pad (fakesink, "sink");
+  gst_pad_add_probe (sinkpad, GST_PAD_PROBE_TYPE_BLOCK, block_probe, NULL,
+      NULL);
+  gst_object_unref (sinkpad);
+
+  /* Set size limit to 1000000 byte, low threshold to 1%, high
+   * threshold to 5%, to make sure that even just one data push
+   * will exceed both thresholds.*/
+  g_object_set (mq,
+      "use-buffering", (gboolean) TRUE,
+      "max-size-bytes", (guint) 1000 * 1000,
+      "max-size-buffers", (guint) 0,
+      "max-size-time", (guint64) 0,
+      "extra-size-bytes", (guint) 0,
+      "extra-size-buffers", (guint) 0,
+      "extra-size-time", (guint64) 0,
+      "low-percent", (gint) 1, "high-percent", (gint) 5, NULL);
+
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+
+  inputpad = gst_pad_new ("dummysrc", GST_PAD_SRC);
+  gst_pad_set_query_function (inputpad, mq_dummypad_query);
+
+  mq_sinkpad = gst_element_get_request_pad (mq, "sink_%u");
+  fail_unless (mq_sinkpad != NULL);
+  fail_unless (gst_pad_link (inputpad, mq_sinkpad) == GST_PAD_LINK_OK);
+
+  gst_pad_set_active (inputpad, TRUE);
+
+  gst_pad_push_event (inputpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (inputpad, gst_event_new_segment (&segment));
+
+  gst_object_unref (mq_sinkpad);
+
+  fail_unless (gst_element_link (mq, fakesink));
+
+  /* Start pipeline in paused state to ensure the sink remains
+   * in preroll mode and blocks */
+  gst_element_set_state (pipe, GST_STATE_PAUSED);
+
+  /* Feed data. queue will be filled to 8% (because it pushes 80000 bytes),
+   * which is above both the low- and the high-threshold. This should
+   * produce a 100% buffering message. */
+  thread = g_thread_new ("push1", pad_push_datablock_thread, inputpad);
+  g_thread_join (thread);
+  check_for_buffering_msg (pipe, 100);
+
+  gst_element_set_state (pipe, GST_STATE_NULL);
+  gst_object_unref (inputpad);
+  gst_object_unref (pipe);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_watermark_and_fill_level)
+{
+  /* This test checks the behavior of the fill level and
+   * the low/high watermarks. It also checks if the
+   * low/high-percent and low/high-watermark properties
+   * are coupled together properly. */
+  GstElement *pipe;
+  GstElement *mq, *fakesink;
+  GstPad *inputpad;
+  GstPad *mq_sinkpad;
+  GstPad *sinkpad;
+  GstSegment segment;
+  GThread *thread;
+  gint low_perc, high_perc;
+
+
+  /* Setup test pipeline with one multiqueue and one fakesink */
+
+  pipe = gst_pipeline_new ("testbin");
+  mq = gst_element_factory_make ("multiqueue", NULL);
+  fail_unless (mq != NULL);
+  gst_bin_add (GST_BIN (pipe), mq);
+
+  fakesink = gst_element_factory_make ("fakesink", NULL);
+  fail_unless (fakesink != NULL);
+  gst_bin_add (GST_BIN (pipe), fakesink);
+
+  /* Block fakesink sinkpad flow to ensure the queue isn't emptied
+   * by the prerolling sink */
+  sinkpad = gst_element_get_static_pad (fakesink, "sink");
+  gst_pad_add_probe (sinkpad, GST_PAD_PROBE_TYPE_BLOCK, block_probe, NULL,
+      NULL);
+  gst_object_unref (sinkpad);
+
+  g_object_set (mq,
+      "use-buffering", (gboolean) TRUE,
+      "max-size-bytes", (guint) 1000 * 1000,
+      "max-size-buffers", (guint) 0,
+      "max-size-time", (guint64) 0,
+      "extra-size-bytes", (guint) 0,
+      "extra-size-buffers", (guint) 0,
+      "extra-size-time", (guint64) 0,
+      "low-watermark", (gdouble) 0.01, "high-watermark", (gdouble) 0.10, NULL);
+
+  g_object_get (mq, "low-percent", &low_perc, "high-percent", &high_perc, NULL);
+
+  /* Check that low/high-watermark and low/high-percent are
+   * coupled properly. (low/high-percent are deprecated and
+   * exist for backwards compatibility.) */
+  fail_unless_equals_int (low_perc, 1);
+  fail_unless_equals_int (high_perc, 10);
+
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+
+  inputpad = gst_pad_new ("dummysrc", GST_PAD_SRC);
+  gst_pad_set_query_function (inputpad, mq_dummypad_query);
+
+  mq_sinkpad = gst_element_get_request_pad (mq, "sink_%u");
+  fail_unless (mq_sinkpad != NULL);
+  fail_unless (gst_pad_link (inputpad, mq_sinkpad) == GST_PAD_LINK_OK);
+
+  gst_pad_set_active (inputpad, TRUE);
+
+  gst_pad_push_event (inputpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (inputpad, gst_event_new_segment (&segment));
+
+  gst_object_unref (mq_sinkpad);
+
+  fail_unless (gst_element_link (mq, fakesink));
+
+  /* Start pipeline in paused state to ensure the sink remains
+   * in preroll mode and blocks */
+  gst_element_set_state (pipe, GST_STATE_PAUSED);
+
+  /* Feed data. queue will be filled to 8% (because it pushes 80000 bytes),
+   * which is below the high-threshold, provoking a buffering message. */
+  thread = g_thread_new ("push1", pad_push_datablock_thread, inputpad);
+  g_thread_join (thread);
+
+  /* Check for the buffering message; it should indicate 80% fill level
+   * (Note that the percentage from the message is normalized) */
+  check_for_buffering_msg (pipe, 80);
+
+  /* Increase the buffer size and lower the watermarks to test
+   * if <1% watermarks are supported. */
+  g_object_set (mq,
+      "max-size-bytes", (guint) 20 * 1000 * 1000,
+      "low-watermark", (gdouble) 0.0001, "high-watermark", (gdouble) 0.005,
+      NULL);
+  /* First buffering message is posted after the max-size-bytes limit
+   * is set to 20000000 bytes & the low-watermark is set. Since the
+   * multiqueue contains 80000 bytes, and the high watermark still is
+   * 0.1 at this point, and the buffer level 80000 / 20000000 = 0.004 is
+   * normalized by 0.1: 0.004 / 0.1 => buffering percentage 4%. */
+  check_for_buffering_msg (pipe, 4);
+  /* Second buffering message is posted after the high-watermark limit
+   * is set to 0.005. This time, the buffer level is normalized this way:
+   * 0.004 / 0.005 => buffering percentage 80%. */
+  check_for_buffering_msg (pipe, 80);
+
+
+  gst_element_set_state (pipe, GST_STATE_NULL);
+  gst_object_unref (inputpad);
+  gst_object_unref (pipe);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_high_threshold_change)
+{
+  /* This test checks what happens if the high threshold is changed to a
+   * value below the current buffer fill level. Expected behavior is for
+   * multiqueue to emit a 100% buffering message in that case. */
+  GstElement *pipe;
+  GstElement *mq, *fakesink;
+  GstPad *inputpad;
+  GstPad *mq_sinkpad;
+  GstPad *sinkpad;
+  GstSegment segment;
+  GThread *thread;
+
+
+  /* Setup test pipeline with one multiqueue and one fakesink */
+
+  pipe = gst_pipeline_new ("testbin");
+  mq = gst_element_factory_make ("multiqueue", NULL);
+  fail_unless (mq != NULL);
+  gst_bin_add (GST_BIN (pipe), mq);
+
+  fakesink = gst_element_factory_make ("fakesink", NULL);
+  fail_unless (fakesink != NULL);
+  gst_bin_add (GST_BIN (pipe), fakesink);
+
+  /* Block fakesink sinkpad flow to ensure the queue isn't emptied
+   * by the prerolling sink */
+  sinkpad = gst_element_get_static_pad (fakesink, "sink");
+  gst_pad_add_probe (sinkpad, GST_PAD_PROBE_TYPE_BLOCK, block_probe, NULL,
+      NULL);
+  gst_object_unref (sinkpad);
+
+  g_object_set (mq,
+      "use-buffering", (gboolean) TRUE,
+      "max-size-bytes", (guint) 1000 * 1000,
+      "max-size-buffers", (guint) 0,
+      "max-size-time", (guint64) 0,
+      "extra-size-bytes", (guint) 0,
+      "extra-size-buffers", (guint) 0,
+      "extra-size-time", (guint64) 0,
+      "low-percent", (gint) 1, "high-percent", (gint) 99, NULL);
+
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+
+  inputpad = gst_pad_new ("dummysrc", GST_PAD_SRC);
+  gst_pad_set_query_function (inputpad, mq_dummypad_query);
+
+  mq_sinkpad = gst_element_get_request_pad (mq, "sink_%u");
+  fail_unless (mq_sinkpad != NULL);
+  fail_unless (gst_pad_link (inputpad, mq_sinkpad) == GST_PAD_LINK_OK);
+
+  gst_pad_set_active (inputpad, TRUE);
+
+  gst_pad_push_event (inputpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (inputpad, gst_event_new_segment (&segment));
+
+  gst_object_unref (mq_sinkpad);
+
+  fail_unless (gst_element_link (mq, fakesink));
+
+  /* Start pipeline in paused state to ensure the sink remains
+   * in preroll mode and blocks */
+  gst_element_set_state (pipe, GST_STATE_PAUSED);
+
+  /* Feed data. queue will be filled to 8% (because it pushes 80000 bytes),
+   * which is below the high-threshold, provoking a buffering message. */
+  thread = g_thread_new ("push1", pad_push_datablock_thread, inputpad);
+  g_thread_join (thread);
+
+  /* Check for the buffering message; it should indicate 8% fill level
+   * (Note that the percentage from the message is normalized, but since
+   * the high threshold is at 99%, it should still apply) */
+  check_for_buffering_msg (pipe, 8);
+
+  /* Set high threshold to half of what it was before. This means that the
+   * relative fill level doubles. As a result, this should trigger a buffering
+   * message with a percentage of 16%. */
+  g_object_set (mq, "high-percent", (gint) 50, NULL);
+  check_for_buffering_msg (pipe, 16);
+
+  /* Set high threshold to a value that lies below the current fill level.
+   * This should trigger a 100% buffering message immediately, even without
+   * pushing in extra data. */
+  g_object_set (mq, "high-percent", (gint) 5, NULL);
+  check_for_buffering_msg (pipe, 100);
+
+  gst_element_set_state (pipe, GST_STATE_NULL);
+  gst_object_unref (inputpad);
+  gst_object_unref (pipe);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_low_threshold_change)
+{
+  /* This tests what happens if the queue isn't currently buffering and the
+   * low-threshold is raised above the current fill level. */
+  GstElement *pipe;
+  GstElement *mq, *fakesink;
+  GstPad *inputpad;
+  GstPad *mq_sinkpad;
+  GstPad *sinkpad;
+  GstSegment segment;
+  GThread *thread;
+
+
+  /* Setup test pipeline with one multiqueue and one fakesink */
+
+  pipe = gst_pipeline_new ("testbin");
+  mq = gst_element_factory_make ("multiqueue", NULL);
+  fail_unless (mq != NULL);
+  gst_bin_add (GST_BIN (pipe), mq);
+
+  fakesink = gst_element_factory_make ("fakesink", NULL);
+  fail_unless (fakesink != NULL);
+  gst_bin_add (GST_BIN (pipe), fakesink);
+
+  /* Block fakesink sinkpad flow to ensure the queue isn't emptied
+   * by the prerolling sink */
+  sinkpad = gst_element_get_static_pad (fakesink, "sink");
+  gst_pad_add_probe (sinkpad, GST_PAD_PROBE_TYPE_BLOCK, block_probe, NULL,
+      NULL);
+  gst_object_unref (sinkpad);
+
+  /* Enable buffering and set the low/high thresholds to 1%/5%. This ensures
+   * that after pushing one data block, the high threshold is reached, and
+   * buffering ceases. */
+  g_object_set (mq,
+      "use-buffering", (gboolean) TRUE,
+      "max-size-bytes", (guint) 1000 * 1000,
+      "max-size-buffers", (guint) 0,
+      "max-size-time", (guint64) 0,
+      "extra-size-bytes", (guint) 0,
+      "extra-size-buffers", (guint) 0,
+      "extra-size-time", (guint64) 0,
+      "low-percent", (gint) 1, "high-percent", (gint) 5, NULL);
+
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+
+  inputpad = gst_pad_new ("dummysrc", GST_PAD_SRC);
+  gst_pad_set_query_function (inputpad, mq_dummypad_query);
+
+  mq_sinkpad = gst_element_get_request_pad (mq, "sink_%u");
+  fail_unless (mq_sinkpad != NULL);
+  fail_unless (gst_pad_link (inputpad, mq_sinkpad) == GST_PAD_LINK_OK);
+
+  gst_pad_set_active (inputpad, TRUE);
+
+  gst_pad_push_event (inputpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (inputpad, gst_event_new_segment (&segment));
+
+  gst_object_unref (mq_sinkpad);
+
+  fail_unless (gst_element_link (mq, fakesink));
+
+  /* Start pipeline in paused state to ensure the sink remains
+   * in preroll mode and blocks */
+  gst_element_set_state (pipe, GST_STATE_PAUSED);
+
+  /* Feed data. queue will be filled to 8% (because it pushes 80000 bytes),
+   * which is above the high-threshold, ensuring that the queue disables
+   * its buffering mode internally. */
+  thread = g_thread_new ("push1", pad_push_datablock_thread, inputpad);
+  g_thread_join (thread);
+
+  /* Check for the buffering message; it should indicate 100% relative fill
+   * level (Note that the percentage from the message is normalized) */
+  check_for_buffering_msg (pipe, 100);
+
+  /* Set low threshold to a 10%, which is above the current fill level of 8%.
+   * As a result, the queue must re-enable its buffering mode, and post the
+   * current relative fill level of 40% (since high-percent is also set to 20%
+   * and 8%/20% = 40%). */
+  g_object_set (mq, "high-percent", (gint) 20, "low-percent", (gint) 10, NULL);
+  check_for_buffering_msg (pipe, 40);
+
+  gst_element_set_state (pipe, GST_STATE_NULL);
+  gst_object_unref (inputpad);
+  gst_object_unref (pipe);
+}
+
+GST_END_TEST;
+
 static gpointer
 pad_push_thread (gpointer data)
 {
@@ -804,7 +1344,7 @@ GST_START_TEST (test_limit_changes)
 
   /* Wait until we are actually blocking... we unfortunately can't
    * know that without sleeping */
-  sleep (1);
+  g_usleep (G_USEC_PER_SEC);
   g_object_set (mq, "max-size-buffers", (guint) 3, NULL);
   g_thread_join (thread);
 
@@ -1059,6 +1599,68 @@ GST_START_TEST (test_initial_events_nodelay)
 
 GST_END_TEST;
 
+static void
+check_for_stream_status_msg (GstElement * pipeline, GstElement * multiqueue,
+    GstStreamStatusType expected_type)
+{
+  GEnumClass *klass;
+  const gchar *expected_nick, *nick;
+  GstMessage *msg;
+  GstStreamStatusType type;
+  GstElement *owner;
+
+  klass = g_type_class_ref (GST_TYPE_STREAM_STATUS_TYPE);
+  expected_nick = g_enum_get_value (klass, expected_type)->value_nick;
+
+  GST_LOG ("waiting for stream-status %s message", expected_nick);
+
+  msg = gst_bus_poll (GST_ELEMENT_BUS (pipeline),
+      GST_MESSAGE_STREAM_STATUS | GST_MESSAGE_ERROR, -1);
+  fail_if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR,
+      "Expected stream-status message, got error message");
+
+  gst_message_parse_stream_status (msg, &type, &owner);
+  nick = g_enum_get_value (klass, type)->value_nick;
+  fail_unless (owner == multiqueue,
+      "Got incorrect owner: %" GST_PTR_FORMAT " expected: %" GST_PTR_FORMAT,
+      owner, multiqueue);
+  fail_unless (type == expected_type,
+      "Got incorrect type: %s expected: %s", nick, expected_nick);
+
+  gst_message_unref (msg);
+  g_type_class_unref (klass);
+}
+
+GST_START_TEST (test_stream_status_messages)
+{
+  GstElement *pipe, *mq;
+  GstPad *pad;
+
+  pipe = gst_pipeline_new ("pipeline");
+  mq = gst_element_factory_make ("multiqueue", NULL);
+
+  gst_bin_add (GST_BIN (pipe), mq);
+
+  pad = gst_element_get_request_pad (mq, "sink_%u");
+  gst_object_unref (pad);
+
+  gst_element_set_state (pipe, GST_STATE_PAUSED);
+
+  check_for_stream_status_msg (pipe, mq, GST_STREAM_STATUS_TYPE_CREATE);
+  check_for_stream_status_msg (pipe, mq, GST_STREAM_STATUS_TYPE_ENTER);
+
+  pad = gst_element_get_request_pad (mq, "sink_%u");
+  gst_object_unref (pad);
+
+  check_for_stream_status_msg (pipe, mq, GST_STREAM_STATUS_TYPE_CREATE);
+  check_for_stream_status_msg (pipe, mq, GST_STREAM_STATUS_TYPE_ENTER);
+
+  gst_element_set_state (pipe, GST_STATE_NULL);
+  gst_object_unref (pipe);
+}
+
+GST_END_TEST;
+
 static Suite *
 multiqueue_suite (void)
 {
@@ -1077,12 +1679,20 @@ multiqueue_suite (void)
    * See https://bugzilla.gnome.org/show_bug.cgi?id=708661 */
   tcase_skip_broken_test (tc_chain, test_output_order);
 
+  tcase_add_test (tc_chain, test_not_linked_eos);
+
   tcase_add_test (tc_chain, test_sparse_stream);
+  tcase_add_test (tc_chain, test_initial_fill_above_high_threshold);
+  tcase_add_test (tc_chain, test_watermark_and_fill_level);
+  tcase_add_test (tc_chain, test_high_threshold_change);
+  tcase_add_test (tc_chain, test_low_threshold_change);
   tcase_add_test (tc_chain, test_limit_changes);
 
   tcase_add_test (tc_chain, test_buffering_with_none_pts);
   tcase_add_test (tc_chain, test_initial_events_nodelay);
 
+  tcase_add_test (tc_chain, test_stream_status_messages);
+
   return s;
 }
 
