@@ -18,7 +18,9 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/gststructure.h>
 #include <gst/check/gstcheck.h>
@@ -159,6 +161,32 @@ GST_START_TEST (test_from_string)
   fail_unless_equals_int (g_value_get_boolean (val), TRUE);
   gst_structure_free (structure);
 
+  /* Tests for flagset deserialisation */
+  s = "foobar,value=0010:ffff";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  fail_unless ((val = gst_structure_get_value (structure, "value")) != NULL);
+  fail_unless (GST_VALUE_HOLDS_FLAG_SET (val));
+  gst_structure_free (structure);
+
+  /* In the presence of the hex values, the strings don't matter as long as they
+   * have the right form */
+  s = "foobar,value=0010:ffff:+random+other/not-the-other";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  fail_unless ((val = gst_structure_get_value (structure, "value")) != NULL);
+  fail_unless (GST_VALUE_HOLDS_FLAG_SET (val));
+  gst_structure_free (structure);
+
+  /* Test that a timecode string is deserialised as a string, not a flagset:
+   * https://bugzilla.gnome.org/show_bug.cgi?id=779755 */
+  s = "foobar,timecode=00:01:00:00";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  fail_unless ((val = gst_structure_get_value (structure, "timecode")) != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (val));
+  gst_structure_free (structure);
+
   s = "0.10:decoder-video/mpeg, abc=(boolean)false";
   ASSERT_CRITICAL (structure = gst_structure_from_string (s, NULL));
   fail_unless (structure == NULL, "Could not get structure from string %s", s);
@@ -228,6 +256,24 @@ GST_START_TEST (test_to_from_string)
 
   gst_structure_free (st1);
   gst_structure_free (st2);
+
+  /* Test NULL strings */
+  st1 = gst_structure_new ("test", "mynullstr", G_TYPE_STRING, NULL, NULL);
+  fail_unless (st1 != NULL);
+  str = gst_structure_to_string (st1);
+  fail_unless (strcmp (str, "test, mynullstr=(string)NULL;") == 0,
+      "Failed to serialize to right string: %s", str);
+
+  st2 = gst_structure_from_string (str, NULL);
+  fail_unless (st2 != NULL);
+  g_free (str);
+
+  fail_unless (gst_structure_is_equal (st1, st2),
+      "Structures did not match:\n\tStructure 1: %" GST_PTR_FORMAT
+      "\n\tStructure 2: %" GST_PTR_FORMAT "\n", st1, st2);
+
+  gst_structure_free (st1);
+  gst_structure_free (st2);
 }
 
 GST_END_TEST;
